
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoEnvironment.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <stdlib.h>
#include <time.h>

#ifdef _MSC_VER
float strtof(const char *nptr, char **endptr)  { return float(strtod(nptr, endptr)); }
#endif


SoSeparator* createTerrain(int argc, char **argv);  // Funkce vytv?j? sc?



// Velikost vkov?apy zv?en? jednu
// (krajina 64x64 se konstruuje ze 65x65 bod)
#define DEFAULT_TERRAIN_DIMENSION  65

// Velikost generovan?rajiny v prostoru
#define DEFAULT_TERRAIN_SIZE 64.f

// Maxim??ka a hloubka krajiny
#define DEFAULT_TERRAIN_HEIGHT_Q  15.f

// Filtrovac?oeficient pro fin??yhlazen?rajiny.
// Vy??o odpov? vy?hrbolatosti",
// ni? v? se to vyfiltruje.
#define DEFAULT_TERRAIN_FILTER_Q  0.5f

// HQ (high quality) ur?e, jakm zpsobem bude rendrov? vsledn?rajina.
// HQ=0 znamen?e kad ?erec bude rendrov?jako dva trojheln?,
// HQ=1 je podobn?edchoz?olb?ale zlom mezi t?to trojheln? je
// vytv?n podle toho, ve kter?sm? je v dan?ladici v??m? vky,
// HQ=2 pid?o stedu ?erce dal?ertex a vyrendruje ?erec ?mi
// trojheln?. Krajina bude m?dvojn?bn?nostv?rojheln?,
// ale stane se hlad?#define DEFAULT_TERRAIN_HQ  0



/*!
  T? pedstavuj? ?erec v krajin?*/
class TerrainPatch {
private:
  void allocHmap(int dimension);
  void freeHmap();
  void filter(float q);
  SbVec3f calculateNormal(const int x, const int y, const float sx, const float sy);

protected:
  float *hmap;
  int dimension;

  inline int getHmapIndex(const int x, const int y);
  int getMaterialIndex(const float height, const float minHeight, const float maxHeight);

public:
  TerrainPatch();
  ~TerrainPatch();

  void generateHeightMap();
  void generateHeightMap(int dimension, float heightQ, float filterQ);

  SoSeparator* getModel();
  SoSeparator* getModel(float size, int hq);

  static SoSeparator* generate();
  static SoSeparator* generate(int dimension, float size, float heightQ, float filterQ, int hq);

  void reset();
};

