/*****************************************************************************\
 *
 * shadow_volume.h
 *
 * Algorithm for shadows ganeration in a scene
 * 
 *  Description: Shadow volume class definition
 *
 *      Authors: Tom Burian (xburia10 _AT stud.fit.vutbr.cz)
 *               
 *    Thanks to: PCJohn (peciva _AT fit.vutbr.cz)
 * 
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef _SHADOW_VOLUME_H_
#define _SHADOW_VOLUME_H_


#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLight.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/manips/SoPointLightManip.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
//
#include <Inventor/fields/SoSFVec3f.h>
#include <Inventor/fields/SoMFVec3f.h>
//
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/actions/SoWriteAction.h>
//
#include <Inventor/SbLinear.h>
#include <Inventor/SoOutput.h>

//----------------------------------------------------------------------------
// ShadowVolume class for creating shadow volumes for one object
//
class ShadowVolume
{
  private:
    SoSeparator * sceneRootNode;    // root node of the scene
    SoSeparator * modelNode;        // shadow casting object
    SoSeparator * shadowRootNode;   // root node of the scene
    SoSeparator * wireRootNode;     // root node of the scene
    SoSeparator * shadowVolumeNode; // shadow volume node
    SoSeparator * wireVolumeNode;   // wire shadow volume node

    // Useful nodes of the scene  
    SoCoordinate3 * coordNode;
    SoNormal * normalNode;
    SbMatrix trMatrix;              // transformation matrix of the object
    SoIndexedTriangleStripSet ** stripNodes;
    SbColor svColor;                // volume color
    bool isModelInGraph;            // if model is not fount in the scene graph
                                    // rooted at sceneRootNode, can't create 
                                    // shadow

  public:
    bool hasNormals;
    bool created;                   // volumes has been created
    float farDepth;                 // shadow volume depth
    int stripsCnt;
    bool showVolumes;               // show wire volume model
    
    // SoMFVec3f * vects2light;     // vectors from object's faces towards light source
                                    // no need to use them  

  public:
    ShadowVolume(SoSeparator * root, SoSeparator * shadow, SoSeparator * wire, SoSeparator * object);
    ~ShadowVolume();    
    
    // computes all shadow volumes of this object (for all lights)
    void createShadowVolumes();

    // compute shadow volume for one light
    void createShadowVolume(SoPointLight * light);

    // delete shadow and wire nodes
    bool clearShadowVolumeNode();
        
    // show wire model of shadow volume
    void showShadowVolume(SoPointLight * light);

    void writeShadowNode(const char *filename);

    void printInfo(const char *msg);
    void printError(const char *msg);

  private:
    bool findTransformationMatrix();
    bool findCoordinates();
    bool findNormals();
    bool findStrips();

    SbVec3f farPoint(SbVec3f lightPos, SbVec3f point);
};

#endif
