/*****************************************************************************\
 *
 * shadows_win.cpp
 *
 * Algorithm for shadows ganeration in a scene
 *
 *  Description: Main WIN32 function
 *
 *      Authors: Tom Burian (xburia10 _AT stud.fit.vutbr.cz)
 *               
 *    Thanks to: PCJohn (peciva _AT fit.vutbr.cz)
 * 
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <stdlib.h>
//
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/SoOutput.h>
//
#include "shadows_win.h"
#include "scene.h"

// Linker: Link with main (not WinMain)
// and use a console
#pragma comment(linker, "/entry:\"mainCRTStartup\"")
#pragma comment(linker, "/subsystem:\"console\"")

extern SoSeparator* createScene(int argc, char **argv);

bool headlight = false;           // Headlight
bool feedbackVisibility = true;   // Coordinate cross

char title[50] = "";
const char *sceneFilename = "scene_graph.iv";


//----------------------------------------------------------------------------
// Checks if stencil buffer is enabled
//
void checkStencilBufferSettings (SoWinExaminerViewer *viewer)
{
  // Check buffer settings
  SbBool stencilB = viewer->getStencilBuffer();
  SbBool doubleB = viewer->isDoubleBuffer();
  SbBool accumB = viewer->getAccumulationBuffer();

  if (stencilB)
    (void)printf("Stencil buffer enabled.\n");
  else
    (void)printf("Stencil buffer disabled.\n");

  return;
}


//----------------------------------------------------------------------------
// Main function
//
int main(int argc, char **argv)
{
  // Initialisation
  HWND window = SoWin::init(argv[0]);
  if (window == NULL) exit(1);

  // Root of the scene
  SoSeparator *root = createScene(argc, argv);
  root->ref();

  ONDEBUG(
    // Debug: write out scene graph into a file "scene_graph.iv"
    SoOutput output;
    output.openFile(sceneFilename);
    SoWriteAction * writer = new SoWriteAction(&output);
    writer->apply(root);
    delete writer;
  );

  // Create window
  SoWinExaminerViewer *viewer = new SoWinExaminerViewer(window);
  viewer->setSceneGraph(root);
  viewer->setHeadlight(headlight);
  viewer->setFeedbackVisibility(feedbackVisibility);  
  sprintf(title, "%s %s", APPNAME, VERSION);
  viewer->setTitle(title);
  // Turn OFF Coin's autoclipping (defaul ON)
  if ( viewer->isAutoClipping() ) {
    viewer->setAutoClipping(FALSE);
    ONDEBUG(printf("AutoClipping ON. Turning OFF.\n"));
  }

  // Enable stencil buffer
  viewer->setStencilBuffer(TRUE);
  ONDEBUG(checkStencilBufferSettings(viewer));

  viewer->show();
 
  // Show window and start render loop
  SoWin::show(window);
  SoWin::mainLoop();

  // Free memory
  delete viewer;
  root->unref();

  return 0;
}

////////////////////////////////////////////////////////////////////////////////
